# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import unittest
from baidu.location_api import LocationApi
from baidu.scheduler import RoundRobinScheduler


class TestLocationApi(unittest.TestCase):
    def setUp(self):
        self.scheduler = RoundRobinScheduler(['wwx6xhe8aQncZZUm7QsIPXKI', '9ea66EnDo1YLFuzu5QDDp4zU'])
        self.location_api = LocationApi(self.scheduler)

    def test_get_detail_location_by_address(self):
        detail = self.location_api.get_detail_location_by_address(u'百度大厦', u'北京')
        self.assertIsNotNone(detail)
        self.assertEqual(detail['precise'], 1)
        self.assertEqual(detail['confidence'], 80)
        self.assertEqual(detail['level'], u'商务大厦')
        self.assertEqual(detail['location']['lng'], 116.30783584945)
        self.assertEqual(detail['location']['lat'], 40.056876296398)

    def test_get_location_by_address(self):
        location = self.location_api.get_location_by_address(u'百度大厦', u'北京')
        self.assertIsNotNone(location)
        self.assertEqual(location['lat'], 40.056876296398)
        self.assertEqual(location['lng'], 116.30783584945)

    def test_get_detail_address_by_location(self):
        detail = self.location_api.get_detail_address_by_location({'lng': 116.322987, 'lat': 39.983424})
        self.assertIsNotNone(detail)
        self.assertEqual(detail['formatted_address'], u'北京市海淀区中关村大街27号1101-08室')
        self.assertEqual(detail['business'], u"中关村,人民大学,苏州街")
        self.assertEqual(detail['addressComponent']['province'], u'北京市')
        self.assertEqual(detail['addressComponent']['city'], u'北京市')

    def test_get_address_by_location(self):
        address = self.location_api.get_address_by_location({'lng': 116.322987, 'lat': 39.983424})
        self.assertIsNotNone(address)
        self.assertEqual(address['province'], u'北京')
        self.assertEqual(address['city'], u'北京')

    def test_get_province_by_location(self):
        province = self.location_api.get_province_by_location({'lng': 116.322987, 'lat': 39.983424})
        self.assertIsNotNone(province)
        self.assertEqual(province, u'北京')

    def test_get_formatted_province(self):
        province = self.location_api.get_formatted_province(u'北京市海淀区百度大厦')
        self.assertIsNotNone(province)
        self.assertEqual(province, u'北京')

    def test_get_formatted_address(self):
        address = self.location_api.get_formatted_address(u'北京市海淀区百度大厦')
        self.assertIsNotNone(address)
        self.assertEqual(address['province'], u'北京')
        self.assertEqual(address['city'], u'北京')

    def test_get_formatted_detail_address(self):
        detail = self.location_api.get_formatted_detail_address(u'北京市海淀区百度大厦')
        self.assertIsNotNone(detail)
        self.assertEqual(detail['addressComponent']['province'], u'北京市')
        self.assertEqual(detail['addressComponent']['city'], u'北京市')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import unittest
from baidu.scheduler import RoundRobinScheduler
from baidu.place_api import PlaceApi
import logging


class TestPlaceApi(unittest.TestCase):
    def setUp(self):
        self.scheduler = RoundRobinScheduler(['wwx6xhe8aQncZZUm7QsIPXKI', '9ea66EnDo1YLFuzu5QDDp4zU'])
        self.place_api = PlaceApi(self.scheduler)

    def test_get_place_by_page(self):
        ret = self.place_api.get_place_by_page(u'银行', u'济南')
        self.assertIsNotNone(ret)
        self.assertEqual(len(ret['results']), 20)

    def test_get_place_all(self):
        ret = self.place_api.get_place_by_page(u'银行', u'济南')
        self.assertIsNotNone(ret)
        total = ret['total']
        logging.debug('total: %d' % total)  # 不起作用
        ret = self.place_api.get_place_all(u'银行', u'济南')
        self.assertIsNotNone(ret)
        self.assertEqual(len(ret), total)

    def test_get_place_by_uids_with_string(self):
        place = self.place_api.get_place_by_uids('c14fc238f7fadd4ea5da390f')
        self.assertIsNotNone(place)
        self.assertEqual(place['name'], u'觉品壹号')
        places = self.place_api.get_place_by_uids(['c14fc238f7fadd4ea5da390f', '5a8fb739999a70a54207c130'])
        self.assertIsNotNone(places)
        self.assertEqual(len(places), 2)
        self.assertEqual(places[1]['name'], u'百度大厦员工食堂')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import json
import logging
from mapapi import baidu
import codecs


map_api = baidu.MapApi()


def get_locs(data):
    for item in data:
        loc = map_api.transform_api.get_place_by_uids(item['uid'])
        if loc:
            item['lat'] = loc['location']['lat']
            item['lng'] = loc['location']['lng']
            item['name'] = loc['name']
            item['address'] = loc['address']
        else:
            logging.error(u'获取地理坐标失败: %s' % item)


def get_locs_from_file(infile):
    import os
    dir_name, ext_name = os.path.splitext(infile)
    outfile = '%s_loc%s' % (dir_name, ext_name)
    with codecs.open(infile, 'r', encoding='utf-8') as f:
        data = json.load(f)
        get_locs(data)
        with codecs.open(outfile, 'w', encoding='utf-8') as f:
            json.dump(data, f)


def run_func(func_name, env, *args):
    if func_name in env:
        func = env.get(func_name)
        if hasattr(func, '__call__'):
            func(*args)
        else:
            print('%s is not a function name' % func_name)
    else:
        print('%s not found' % func_name)


if __name__ == '__main__':
    import sys

    if len(sys.argv) < 2:
        print("error, not less than one parameter")
        exit(-1)
    run_func(sys.argv[1], globals(), *sys.argv[2:])



# -*- coding: utf-8 -*-
import codecs
from mapapi import baidu
import json

with codecs.open('data/clinic.txt', 'r', encoding='utf-8') as f:
    map_api = baidu.MapApi()
    matched = []
    unmatched = []
    for line in f:
        clinic_name = line.strip()
        loc = map_api.location_api.get_location_by_address(u'深圳 %s' % clinic_name)
        if loc:
            matched.append({'name': clinic_name, 'loc': loc})
        else:
            unmatched.append(clinic_name)

    print('nmatched: %d, unmatched: %d' % (len(matched), len(unmatched)))

    with codecs.open('data/matched_clinic.txt', 'w', encoding='utf-8') as f:
        json.dump(matched, f)



# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import requests
import logging
import json


class TransformApi(object):
    """
    transform other format coordinates to baidu coordinates.
    """
    transform_url = 'http://api.map.baidu.com/geoconv/v1/'

    def __init__(self, scheduler):
        self.scheduler = scheduler

    def _stringify_coords(self, coords):
        if isinstance(coords, dict):
            return '%s,%s' % (coords['lng'], coords['lat'])

        coords_str = ''
        for coord in coords:
            coords_str += '%s,%s;' % (coord['lng'], coord['lat'])
        coords_str = coords_str[:-1]
        return coords_str

    def _format_result(self, result):
        ret = []
        for item in result:
            ret.append({'lng': item['x'], 'lat': item['y']})
        return ret

    def transform(self, coords, **kwargs):
        """
        完成坐标转换

        :param coords: {'lat':, 'lng':, }或者list of {'lat':, 'lng':,}
        :param kwargs: available parameters include 'from', 'to', 'output'
        :return: if success  return
            [
                { 'lat':, 'lng':, }
                ...
            ]
            else return None.
        """
        from_type = kwargs.get('from', 3)
        to_type = kwargs.get('to', 5)
        output = kwargs.get('output', 'json')
        params = {'from': from_type, 'to': to_type, 'ak': self.scheduler.next(),
                  'output': output, 'coords': self._stringify_coords(coords)}
        try:
            r = requests.get(self.transform_url, params=params)
            r.raise_for_status()

            data = json.loads(r.text)
            if data['status'] == 0:
                return self._format_result(data['result'])
            logging.debug(u'failed to transform: %s' % r.text)
            return None
        except:
            logging.exception(u'获取坐标转化失败: %s' % json.dumps(params))
        return None



# -*- coding: utf-8 -*-
""" use as main map interface. """

from mapapi.baidu.scheduler import RoundRobinScheduler
from mapapi.baidu.location_api import LocationApi
from mapapi.baidu.place_api import PlaceApi
from mapapi.baidu.transform_api import TransformApi


class MapApi(object):
    def __init__(self, aks=None, scheduler=None, **kwargs):
        """
        initialize MapApi instance with application keys `aks`, aks scheduler.

        :param aks: list of application keys.
        :param scheduler: default RoundRobinScheduler
        :param kwargs:
        """
        super(MapApi, self).__init__()
        self.aks = aks or ['wwx6xhe8aQncZZUm7QsIPXKI', '9ea66EnDo1YLFuzu5QDDp4zU']
        self.scheduler = scheduler
        self._place_api_inst = None
        self._location_api_inst = None
        self._transform_api_inst = None

    def set_aks(self, aks):
        """
        set application keys.

        :param aks: list of application keys
        """
        self.aks = aks

    def set_scheduler(self, scheduler):
        """
        set scheduler.

        :param scheduler:
        """
        self.scheduler = scheduler

    @property
    def place_api(self):
        """
        get place api interface.

        :return: PlaceApi instance.
        """
        if self._place_api_inst:
            return self._place_api_inst
        if not self.scheduler:
            self.scheduler = RoundRobinScheduler(self.aks)
        self._place_api_inst = PlaceApi(self.scheduler)
        return self._place_api_inst

    @property
    def location_api(self):
        """
        get location api interface.

        :return: LocationApi instance.
        """
        if self._location_api_inst:
            return self._location_api_inst
        if not self.scheduler:
            self.scheduler = RoundRobinScheduler(self.aks)
        self._location_api_inst = LocationApi(self.scheduler)
        return self._location_api_inst

    @property
    def transform_api(self):
        """
        get transform api interface.

        :return: TransformApi instance.
        """
        if self._transform_api_inst:
            return self._transform_api_inst
        if not self.scheduler:
            self.scheduler = RoundRobinScheduler(self.aks)
        self._transform_api_inst = TransformApi(self.scheduler)
        return self._transform_api_inst



# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import unittest
from baidu.scheduler import RoundRobinScheduler
from baidu.exceptions import ParamError


class TestRoundRobinScheduler(unittest.TestCase):
    def test_next_with_correct_aks(self):
        scheduler = RoundRobinScheduler(['wwx6xhe8aQncZZUm7QsIPXKI', '9ea66EnDo1YLFuzu5QDDp4zU'])
        self.assertEqual(scheduler.next(), 'wwx6xhe8aQncZZUm7QsIPXKI')
        self.assertEqual(scheduler.next(), '9ea66EnDo1YLFuzu5QDDp4zU')
        self.assertEqual(scheduler.next(), 'wwx6xhe8aQncZZUm7QsIPXKI')

    def test_next_with_empty_aks(self):
        scheduler = RoundRobinScheduler([])
        with self.assertRaises(ParamError):
            scheduler.next()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
u""" 处理城市坐标 """
__author__ = 'zhangjinjie'

import requests
import logging
import json
from mapapi.baidu.exceptions import ParamError

# 返回码
baidu_ret_code = {
    0: u'正常',
    1: u'服务器内部错误',
    2: u'请求参数非法',
    3: u'权限校验失败',
    4: u'配额校验失败',
    5: u'ak不存在或者非法',
    101: u'服务禁用',
    102: u'不通过白名单或者安全码不对',
}


class LocationApi(object):
    """
    translate address to geographical coordinates(refer to location) or location to address....
    """
    # 通过地址获取地理坐标url地址
    baidu_json_location_url = u'http://api.map.baidu.com/geocoder/v2/'

    # 通过地理坐标返回地址信息url地址
    baidu_json_region_url = u'http://api.map.baidu.com/geocoder/v2/'

    def __init__(self, scheduler):
        self.scheduler = scheduler

    def _format_province(self, province):
        u"""
        标准化省份信息, 比如将'内蒙古自治区'统一转换成'内蒙古'.

        :param province: origin province
        :return: formatted province
        """
        length = len(province)
        if province.endswith(u'省') or province.endswith(u'市'):
            return province[0:length - 1]
        elif province.endswith(u'自治区'):
            if u'内蒙古' in province:
                return u'内蒙古'
            if u'宁夏' in province:
                return u'宁夏'
            if u'广西' in province:
                return u'广西'
            if u'新疆' in province:
                return u'新疆'
            if u'西藏' in province:
                return u'西藏'
        return province

    def _format_city(self, city):
        """
        标准化城市信息, 去掉城市尾部所带的'市'.

        :param city: origin city
        :return: formatted city
        """
        if u'市' in city:
            length = len(city)
            return city[0:length - 1]
        return city

    def get_detail_location_by_address(self, address, city=None):
        """
        根据地址信息获取地理坐标

        :param address: address must include province and city information.
        :param city: city just functions as a filter for return data.
        :return: if success, return
            { 'location':  # 经纬度坐标
                { 'lat': , 'lng': },
              'precise': , # 位置的附加信息, 是否精确查找, 1为精确查找, 0为不精确
              'confidence': , # 可信度
              'level': , # 地址类型
            }
            else return None.
        """
        if address is None:
            raise ParamError(u'address不能为空')
        params = {'address': address, 'ak': self.scheduler.next(), 'output': 'json'}
        if city:
            params['city'] = city
        try:
            r = requests.get(self.baidu_json_location_url, params=params)
            r.raise_for_status()

            data = json.loads(r.text)
            if data['status'] == 0:
                return data['result']
            else:
                logging.debug(r.text)
        except Exception as e:
            logging.exception(e)

        logging.error(u'failed to get address location for %s' % address)
        return None

    def get_location_by_address(self, address, city=None):
        """
        Same with get_detail_location_by_address, except for just return {'lat': , 'lng'} location info.

        :param address:
        :param city:
        :return: if success return {'lat': , 'lng': } else return None.
        """
        detail = self.get_detail_location_by_address(address, city)
        if detail:
            return detail['location']
        return None

    def get_detail_address_by_location(self, location, **kwargs):
        """
        根据地理坐标获取地址信息

        :param location: {'lat': , 'lng': } 地理坐标
        :param kwargs:
        :return: if success return
            { 'location': {'lat':, 'lng'},
              'origin_location': {'lat':, 'lng'}, # 这是请求的location, 并非百度原始返回的
              'formatted_address":, # 结构化地址信息
              'business':, # 商圈
              'addressComponent': # 地址信息
                { 'country":, # 国家
                  'province':, # 省名
                  'city':, # 城市名
                  'district':, # 区县名
                  'street':, # 街道名
                  'street_number':, # 街道门牌号
                  'country_code':, # 国家code
                  'direction':, # 和当前坐标点的方向
                  'distance': # 和当前坐标点的距离
                }
              'pois': # 周边poi数组
                { 'addr':, # 地址信息
                  'cp':, # 数据来源
                  'direction':, # 和当前坐标点方向
                  'distance':, # 离坐标点距离
                  'name':, # poi名称
                  'poiType':, # poi类型, 如'办公大楼, 商务大厦'
                  'point':, # poi坐标
                  'tel':, # 电话
                  'uid':, # poi唯一标识
                  'zip':, # 邮编
                }
              'sematic_description':, # 当前位置结合POI的语义化结果描述
            } or return None.
        :raise ParamError: if location is None.
        """
        if location is None:
            raise ParamError(u'location不能为空')
        params = {'ak': self.scheduler.next(), 'output': 'json', 'location': '{lat},{lng}'.format(lat=location['lat'],
                                                                                                  lng=location['lng']),
                  'coordtype': kwargs.get('coordtype', 'bd09ll'), 'pois': kwargs.get('pois', 0)}
        try:
            r = requests.get(self.baidu_json_region_url, params=params)
            r.raise_for_status()

            data = json.loads(r.text)
            if data['status'] == 0:
                data['result']['origin_location'] = {'lat': location['lat'], 'lng': location['lng']}
                return data['result']
            else:
                logging.debug(r.text)
        except Exception as e:
            logging.exception(e)

        logging.error(u'failed to get province for %s' % str(location))
        return None

    def get_province_by_location(self, location, **kwargs):
        """
        Same with get_detail_address_by_location, except for just returning formatted province info.

        :param location: {'lat':, 'lng':,}
        :param kwargs:
        :return: if success, return formatted province, else return None.
        """
        detail = self.get_detail_address_by_location(location, **kwargs)
        if detail:
            return self._format_province(detail['addressComponent']['province'])
        return None

    def get_address_by_location(self, location, **kwargs):
        """
        Same with get_detail_address_by_location, except for just returning addressComponent info whose provine and city
        are formatted.

        :param location: {'lat':, 'lng':}
        :param kwargs:
        :return: if success return
                { 'country":, # 国家
                  'province':, # 省名
                  'city':, # 城市名
                  'district':, # 区县名
                  'street':, # 街道名
                  'street_number':, # 街道门牌号
                  'country_code':, # 国家code
                  'direction':, # 和当前坐标点的方向
                  'distance': # 和当前坐标点的距离
                }
                else return None.
        """
        detail = self.get_detail_address_by_location(location, **kwargs)
        if detail:
            address = detail['addressComponent']
            address['province'] = self._format_province(address['province'])
            address['city'] = self._format_province(address['city'])
            return address
        return None

    def get_formatted_province(self, address):
        """
        返回地址所对应的标准化省份名称

        :param address: 详细地址信息
        :return: if success return province info else return None.
        """
        location = self.get_location_by_address(address)
        if location:
            return self.get_province_by_location(location)
        return None

    def get_formatted_address(self, address):
        """
        返回地址所对应的标准化地址信息

        :param address: 详细地址信息
        :return: if success return
                { 'country":, # 国家
                  'province':, # 省名
                  'city':, # 城市名
                  'district':, # 区县名
                  'street':, # 街道名
                  'street_number':, # 街道门牌号
                  'country_code':, # 国家code
                  'direction':, # 和当前坐标点的方向
                  'distance': # 和当前坐标点的距离
                }
                else return None.
        """
        location = self.get_location_by_address(address)
        if location:
            return self.get_address_by_location(location)
        return None

    def get_formatted_detail_address(self, address):
        """
        返回地址所对应的标准化的详细地址

        :param address: 详细地址信息
        :return: if success return
            { 'location': {'lat':, 'lng'},
              'origin_location': {'lat':, 'lng'}, # 这是请求的location, 并非百度原始返回的
              'formatted_address":, # 结构化地址信息
              'business':, # 商圈
              'addressComponent': # 地址信息
                { 'country":, # 国家
                  'province':, # 省名
                  'city':, # 城市名
                  'district':, # 区县名
                  'street':, # 街道名
                  'street_number':, # 街道门牌号
                  'country_code':, # 国家code
                  'direction':, # 和当前坐标点的方向
                  'distance': # 和当前坐标点的距离
                }
              'pois': # 周边poi数组
                { 'addr':, # 地址信息
                  'cp':, # 数据来源
                  'direction':, # 和当前坐标点方向
                  'distance':, # 离坐标点距离
                  'name':, # poi名称
                  'poiType':, # poi类型, 如'办公大楼, 商务大厦'
                  'point':, # poi坐标
                  'tel':, # 电话
                  'uid':, # poi唯一标识
                  'zip':, # 邮编
                }
              'sematic_description':, # 当前位置结合POI的语义化结果描述
            } or return None.
        """
        location = self.get_location_by_address(address)
        if location:
            return self.get_detail_address_by_location(location)
        return None




# -*- coding: utf-8 -*-
"""
抓取广东省主要城市大中型超市, 便利店, 粤菜分布
"""

from xpinyin import Pinyin
from mapapi import baidu
import json
import logging
import re

malls = [u'家乐福', u'沃尔玛', u'好又多']
cities = [u'广州', u'深圳', u'佛山', u'东莞', u'惠州', u'中山', u'湛江', u'茂名', u'江门']

piny = Pinyin()
map_api = baidu.MapApi()


def get_file_name_by_city(city):
    return 'data/%s.json' % (piny.get_pinyin(city, ''))


def get_malls_by_city(malls, city, filter=None):
    """

    :param malls:
    :param city:
    :param filter: 过滤函数func(item, mall, city), 返回值为False的item将会被过滤掉
    """
    file_name = get_file_name_by_city(city)
    with open(file_name, 'w') as f:
        locations = []
        for m in malls:
            data = map_api.place_api.get_place_all(m, city, scope=2, industry_type='life', groupon=0, discount=0)
            nfilter = 0
            for item in data:
                try:
                    if filter and not filter(item, m, city):
                        nfilter += 1
                        continue
                    locations.append({'lat': item['location']['lat'], 'lng': item['location']['lng'], 'name': item['name'], 'keyword': m})
                except:
                    logging.exception(u'item格式错误: %s' % json.dumps(item))
                    print(item)
            if filter:
                print(city, m, len(data) - nfilter, 'filtered: %d' % nfilter)
            else:
                print(city, m, len(data))
        json.dump(locations, f)


def get_malls_all(filter=None):
    for city in cities:
        get_malls_by_city(malls, city, filter)


def filter_by_name(item, mall, city):
    pattern = u'\(%s.*店\)' % mall
    match = re.search(pattern, item['name'])
    if match:
        return False
    pattern = u'%s.*' % mall
    match = re.match(pattern, item['name'])
    if match:
        return True
    return False


def run_func(func_name, env, *args):
    if func_name in env:
        func = env.get(func_name)
        if hasattr(func, '__call__'):
            func(*args)
        else:
            print('%s is not a function name' % func_name)
    else:
        print('%s not found' % func_name)


if __name__ == '__main__':
    import sys

    if len(sys.argv) < 2:
        print("error, not less than one parameter")
        exit(-1)
    run_func(sys.argv[1], globals(), *sys.argv[2:])



# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import unittest
from test_location_api import TestLocationApi
from test_place_api import TestPlaceApi
from test_scheduler import TestRoundRobinScheduler
from test_transform_api import TestTransformApi
from test_map_api import TestMapApi

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import requests
import json
import logging


class PlaceApi(object):
    u"""
    search poi data by keyword.
    """
    search_url = 'http://api.map.baidu.com/place/v2/search'
    detail_url = 'http://api.map.baidu.com/place/v2/detail'
    eventsearch_url = 'http://api.map.baidu.com/place/v2/eventsearch'
    eventdetail_url = 'http://api.map.baidu.com/place/v2/eventdetail'

    def __init__(self, scheduler):
        self.scheduler = scheduler

    def get_place_by_page(self, query, region, **kwargs):
        u"""
        城市内检索

        百度在没有查找到对应查询请求时, 会返回在其它城市查找到的结果, 返回格式为[{'num': , 'name': ''} ...]这样的数组
        获取一页query相关地理信息
        :param query: 查询关键词
        :param region: 地区
        :param kwargs:
        :return:  if success return
            {
                status: 本次API访问状态, 成功返回0, 其他返回其他数字,
                message: 对本次API访问状态值的英文说明, 如果成功返回'ok', 失败返回错误说明,
                total: 检索总数, 用户请求中设置了page_num字段时才会出现, 当检索总数超过760时, 多次刷新同一请求得到的total值, 可能稍有不同
                results: [
                    {
                        name:  POI名称,
                        location: {
                            lat: 纬度,
                            lng: 经度
                        },
                        address: POI地址信息,
                        telephone: POI电话信息,
                        uid: POI的唯一标识,
                        detail_info: {  # POI扩展信息, 仅当scope=2时, 显示该字段, 不同POI类型, 显示的detail_info字段不同
                            distance: 距离中心点距离,
                            type: POI类型,
                            tag: 标签,
                            detail_url: POI的详情页,
                            price: POI商户的价格,
                            shop_hours: 营业时间,
                            overall_rating: 总体评分,
                            taste_rating: 口味评分,
                            service_rating: 服务评分,
                            environment_rating: 环境评分,
                            facility_rating: 星级评分,
                            hygiene_rating: 卫生评分,
                            technology_rating: 技术评分,
                            image_num: 图片数,
                            groupon_num: 团购数,
                            discount_num: 优惠数,
                            comment_num: 评论数,
                            favorite_num: 收藏数,
                            checkin_num: 签到数
                        }
                    }
                    ...
                ]
            }
            else return None.
        """
        tag = kwargs.get('tag', '')
        scope = kwargs.get('scope', 1)  # 检索结果详细成都, 1 基本信息, 2 POI详细信息
        # filter字段设置, scope为2时有效
        industry_type = kwargs.get('industry_type', 'cater')  # 行业类型. 取值范围为: hotel 宾馆, cater 餐饮, life 生活娱乐
        # 排序字段. industry_type为hotel时, 取指范围为: default 默认, price 价格, total_score 好评, level: 星级,
        # health_score: 卫生, distance: 距离; 为cater时, default: 默认, taste_rating: 口味, price: 价格,
        # overall_rating: 好评, service_rating: 服务, distance: 距离; 为life时, default: 默认, price: 价格,
        # overall_rating: 好评, comment_num: 服务, distance: 距离
        sort_name = kwargs.get('sort_name', 'default')
        sort_rule = kwargs.get('sort_rule', 0)  # 排序规则, 0 从高到低, 1 从低到高
        groupon = kwargs.get('groupon', 1)  # 是否有团购, 1 有团购, 0 无团购
        discount = kwargs.get('discount', 1)  # 是否有打折, 1 有打折, 0 无打折
        page_size = kwargs.get('page_size', 20)  # 每页数据记录数. 最大返回20条
        page_num = kwargs.get('page_num', 0)  # 页序号
        params = {'query': query, 'output': 'json', 'scope': scope, 'page_size': page_size, 'page_num': page_num,
                  'ak': self.scheduler.next()}
        if scope == 2:
            filter = 'industry_type:{industry_type}|sort_name:{sort_name}|sort_rule:{sort_rule}|groupon:{groupon}|' \
                     'discount:{discount}'.format(industry_type=industry_type, sort_name=sort_name,
                                                  sort_rule=sort_rule, groupon=groupon, discount=discount)
            params['filter'] = filter

        if tag:
            params['tag'] = tag

        params['region'] = region
        r = requests.get(self.search_url, params=params)
        try:
            r.raise_for_status()
            data = json.loads(r.text)
            # print json.dumps(data, ensure_ascii=False)
            if data['status'] == 0:
                # 在状态为0时, 也有可能没有找到搜索结果, 而是返回在其它城市查找到的结果, 返回格式为[{'num': , 'name': ''} ...]这样的数组
                if len(data['results']) > 0:
                    if 'location' in data['results'][0]:
                        return data
                    logging.debug(data['results'])
                    return None
                return data
            else:
                logging.error('failed to get place, return result is %s' % r.text)
                return None
        except Exception as e:
            logging.exception(e)
            return None

    def get_place_all(self, query, region, **kwargs):
        u"""
        根据关键词query查找所有地址信息

        *注意* 百度最多返回400条记录
        :param query: 查询关键词
        :param region: 地区
        :param kwargs:
        :return: if success return
            [
                {
                    name:  POI名称,
                    location: {
                        lat: 纬度,
                        lng: 经度
                    },
                    address: POI地址信息,
                    telephone: POI电话信息,
                    uid: POI的唯一标识,
                    detail_info: {  # POI扩展信息, 仅当scope=2时, 显示该字段, 不同POI类型, 显示的detail_info字段不同
                        distance: 距离中心点距离,
                        type: POI类型,
                        tag: 标签,
                        detail_url: POI的详情页,
                        price: POI商户的价格,
                        shop_hours: 营业时间,
                        overall_rating: 总体评分,
                        taste_rating: 口味评分,
                        service_rating: 服务评分,
                        environment_rating: 环境评分,
                        facility_rating: 星级评分,
                        hygiene_rating: 卫生评分,
                        technology_rating: 技术评分,
                        image_num: 图片数,
                        groupon_num: 团购数,
                        discount_num: 优惠数,
                        comment_num: 评论数,
                        favorite_num: 收藏数,
                        checkin_num: 签到数
                    }
                }
                ...
            ]
            else return []
        """
        data = []
        kwargs.update({'page_num': 0})
        r = self.get_place_by_page(query, region, **kwargs)
        if r is None:
            return data
        data.extend(r['results'])
        total = r['total']
        page_size = kwargs.get('page_size', 20)
        # print "total: %d, page_size: %d" % (total, page_size)
        for i in range(1, total / page_size + 1):
            kwargs.update({'page_num': i})
            r = self.get_place_by_page(query, region, **kwargs)
            if r is None:
                break
            if r['total'] == 0:
                break
            data.extend(r['results'])
        return data

    def get_place_by_uids(self, uids, **kwargs):
        u"""
        Place详情检索服务

        uids最多支持10个
        :param uids: string or list
        :param kwargs: available keys include 'output', 'scope'
        :return: same with get_place_all.
        """
        params = {}
        if isinstance(uids, list):
            params['uids'] = ','.join(uids)
        else:
            params['uid'] = uids
        params['output'] = kwargs.get('output', 'json')  # json or xml 请求返回格式
        params['scope'] = kwargs.get('scope', 1)  # 1 返回基本信息, 2 返回POI详细信息
        params['ak'] = self.scheduler.next()
        try:
            r = requests.get(self.detail_url, params=params)
            r.raise_for_status()

            data = json.loads(r.text)
            if data['status'] == 0:
                return data['result']

            logging.error('failed to get place, return result is %s' % r.text)
            return []
        except Exception as e:
            logging.exception(e)
        return []



# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import unittest
from baidu import MapApi


class TestMapApi(unittest.TestCase):
    def setUp(self):
        self.map_api = MapApi(['wwx6xhe8aQncZZUm7QsIPXKI', '9ea66EnDo1YLFuzu5QDDp4zU'])

    def test_location_api(self):
        detail = self.map_api.location_api.get_detail_location_by_address(u'百度大厦', u'北京')
        self.assertIsNotNone(detail)
        self.assertEqual(detail['precise'], 1)
        self.assertEqual(detail['confidence'], 80)
        self.assertEqual(detail['level'], u'商务大厦')
        self.assertEqual(detail['location']['lng'], 116.30783584945)
        self.assertEqual(detail['location']['lat'], 40.056876296398)

    def test_place_api(self):
        ret = self.map_api.place_api.get_place_by_page(u'银行', u'济南')
        self.assertIsNotNone(ret)
        self.assertEqual(len(ret['results']), 20)

    def test_transform_api(self):
        coords = self.map_api.transform_api.transform({'lat': 29.5754297789, 'lng': 114.218927345})
        self.assertIsNotNone(coords)
        self.assertEqual(len(coords), 1)
        self.assertEqual(coords[0]['lat'], 29.58158536743)
        self.assertEqual(coords[0]['lng'], 114.22539195408)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
__author__ = 'zhangjinjie'

import unittest
from baidu.scheduler import RoundRobinScheduler
from baidu.transform_api import TransformApi


class TestTransformApi(unittest.TestCase):
    def setUp(self):
        self.scheduler = RoundRobinScheduler(['wwx6xhe8aQncZZUm7QsIPXKI', '9ea66EnDo1YLFuzu5QDDp4zU'])
        self.transform_api = TransformApi(self.scheduler)

    def test_transform_with_single_param(self):
        # python 浮点数精度有限
        coords = self.transform_api.transform({'lat': 29.5754297789, 'lng': 114.218927345})
        self.assertIsNotNone(coords)
        self.assertEqual(len(coords), 1)
        self.assertEqual(coords[0]['lat'], 29.58158536743)
        self.assertEqual(coords[0]['lng'], 114.22539195408)

    def test_transform_with_list(self):
        coords = self.transform_api.transform([{'lat': 29.5754297789, 'lng': 114.218927345},
                                               {'lat': 29.5754297789, 'lng': 114.218927345}])
        self.assertIsNotNone(coords)
        self.assertEqual(len(coords), 2)
        self.assertEqual(coords[1]['lat'], 29.58158536743)
        self.assertEqual(coords[1]['lng'], 114.22539195408)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from mapapi.baidu.exceptions import ParamError


class BaseScheduler(object):
    """
    use as scheduler base class.
    """
    def __init__(self, aks):
        super(BaseScheduler, self).__init__()
        self.aks = aks

    def next(self):
        raise NotImplementedError


class RoundRobinScheduler(BaseScheduler):
    """
    implement round-robin poll scheduler algorithm
    """
    def __init__(self, aks):
        super(RoundRobinScheduler, self).__init__(aks)
        self.cur_index = 0

    def next(self):
        if len(self.aks) < 1:
            raise ParamError(u'aks不能为空')
        ak = self.aks[self.cur_index]
        self.cur_index += 1
        if self.cur_index == len(self.aks):
            self.cur_index = 0
        return ak



from setuptools import setup
import codecs

def readme():
    with codecs.open('README.rst', 'r', encoding='utf-8') as f:
        return f.read()

setup(name='mapapi',
      version='0.2.2',
      description='map web api, current support baidu',
      long_description=readme(),
      keywords='map baidu',
      url='https://github.com/starplanet/mapapi',
      author='zhangjinjie',
      author_email='zhangjinjie@yimian.com.cn',
      license='MIT',
      packages=['mapapi', 'mapapi/baidu'],
      install_requires=[
          'requests',
      ],
      include_package_data=True,
      zip_safe=False)
